<section id="sec-engine" class="tab-section">
    <div class="config-section-hero">
        <h2 class="config-section-hero__title">Engine & Processes</h2>
        <p class="config-section-hero__desc">Active channels, system nodes, sessions, and scheduled tasks.</p>
    </div>

    <!-- Nodes -->
    <div id="panel-nodes" class="config-section-card" hx-get="/dashboard/partials/nodes__TOKEN_SUFFIX__"
        hx-trigger="load" hx-swap="outerHTML">
        <div style="padding:18px; color:var(--muted); font-size:12px;">Loading nodes...</div>
    </div>

    <!-- Two-column: Sessions + Channels -->
    <div class="kb-grid-2">
        <div id="panel-sessions" class="config-section-card" hx-get="/dashboard/partials/sessions__TOKEN_SUFFIX__"
            hx-trigger="load" hx-swap="outerHTML">
            <div style="padding:18px; color:var(--muted); font-size:12px;">Loading sessions...</div>
        </div>
        <div class="config-section-card" hx-get="/dashboard/partials/channels__TOKEN_SUFFIX__"
            hx-trigger="load" hx-swap="innerHTML">
            <div style="padding:18px; color:var(--muted); font-size:12px;">Loading channels...</div>
        </div>
    </div>

    <!-- Cron Jobs -->
    <div id="panel-cron" class="config-section-card" hx-get="/dashboard/partials/cron__TOKEN_SUFFIX__"
        hx-trigger="load" hx-swap="outerHTML">
        <div style="padding:18px; color:var(--muted); font-size:12px;">Loading cron jobs...</div>
    </div>

    <!-- Sub-Agent Activity -->
    <div id="panel-subagents" class="config-section-card" hx-get="/dashboard/partials/subagents__TOKEN_SUFFIX__"
        hx-trigger="load" hx-swap="outerHTML">
        <div style="padding:18px; color:var(--muted); font-size:12px;">Loading sub-agent activity...</div>
    </div>
</section>
